@@ -6,11 +6,11 @@
     :copyright: (c) 2014 by Taehoon Kim.
     :license: BSD, see LICENSE for more details.
 """
-
+from __future__ import print_function
 import re
 import requests
+import itertools
 from datetime import datetime
-from contants import *
 
 try:
     import simplejson as json
@@ -43,7 +43,6 @@ KORAIL_EVENT            = "%s.common.event" % KORAIL_MOBILE
 KORAIL_PAYMENT          = "%s/ebizmw/PrdPkgMainList.do" % KORAIL_DOMAIN
 KORAIL_PAYMENT_VOUNCHER = "%s/ebizmw/PrdPkgBoucherView.do" % KORAIL_DOMAIN
 
-
 class Schedule(object):
     """Korail train object. Highly inspired by `korail.py
     <https://raw.githubusercontent.com/devxoul/korail/master/korail/korail.py>`_
@@ -89,7 +88,7 @@ class Schedule(object):
     arr_code = None # h_arv_rs_stn_cd
 
     #: 도착 날짜 (yyyyMMdd)
-    dep_date = None # h_arv_dt
+    arr_date = None # h_arv_dt
 
     #: 도착 시각 (hhmmss)
     arr_time = None # h_arv_tm
@@ -148,13 +147,13 @@ class Train(Schedule):
     #: 00: 특실 없음
     #: 11: 예약 가능
     #: 13: 매진
-    special_seat = False # h_spe_rsv_cd
+    special_seat = None # h_spe_rsv_cd
 
     #: 일반실 예약가능 여부
     #: 00: 일반실 없음
     #: 11: 예약 가능
     #: 13: 매진
-    general_seat = False # h_gen_rsv_cd
+    general_seat = None # h_gen_rsv_cd
 
     def __init__(self, data):
         super(Train, self).__init__(data)
@@ -169,15 +168,24 @@ class Train(Schedule):
 
         if self.reserve_possible_name is not None:
             seats = []
-            if self.special_seat == '11':
+            if self.has_special_seat():
                 seats.append("특실")
 
-            if self.general_seat == '11':
+            if self.has_general_seat():
                 seats.append("일반실")
+
             repr_str += " "+ (",".join(seats)) + " " + self.reserve_possible_name.replace('\n',' ')
 
         return repr_str
 
+    def has_special_seat(self):
+        return self.special_seat == '11'
+
+    def has_general_seat(self):
+        return self.general_seat == '11'
+
+    def has_seat(self):
+        return self.has_general_seat() or self.has_special_seat()
 
 class Ticket(Train):
     """Ticket object"""
@@ -258,7 +266,19 @@ class Passenger:
     card_no = ''        # txtCardNo_1    : '',    #할인카드 번호
     card_pw = ''        # txtCardPw_1    : '',    #할인카드 비밀번호
 
-    def __init__(self, typecode=None, count=1, discount_type='000', card='', card_no='', card_pw=''):
+    @staticmethod
+    def reduce(passenger_list):
+        """Reduce passenger's list."""
+        if filter(lambda x: not isinstance(x, Passenger), passenger_list):
+            raise TypeError("Passengers must be based on Passenger")
+
+        groups = itertools.groupby(passenger_list, lambda x: x.group_key())
+        return filter(lambda x: x.count > 0, [reduce(lambda a, b: a + b, g) for k, g in groups])
+
+    def __init__(self, *args, **kwargs):
+        raise NotImplementedError("Passenger is abstract class. Do not make instance.")
+
+    def __init_internal__(self, typecode, count=1, discount_type='000', card='', card_no='', card_pw=''):
         self.typecode = typecode
         self.count = count
         self.discount_type = discount_type
@@ -266,6 +286,19 @@ class Passenger:
         self.card_no = card_no
         self.card_pw = card_pw
 
+    def __add__(self, other):
+        assert isinstance(other, self.__class__)
+        if self.group_key() == other.group_key():
+            return self.__class__(count=self.count + other.count, discount_type=self.discount_type, card=self.card,
+                                  card_no=self.card_no, card_pw=self.card_pw)
+        else:
+            raise TypeError(
+                "other's group_key(%s) is not equal to self's group_key(%s)." % (other.group_key(), self.group_key()))
+
+    def group_key(self):
+        """get group string from attributes except count"""
+        return "%s_%s_%s_%s_%s" % (self.typecode, self.discount_type, self.card, self.card_no, self.card_pw)
+
     def get_dict(self, index):
         assert isinstance(index,int)
         index = str(index)
@@ -281,17 +314,43 @@ class Passenger:
 
 class AdultPassenger(Passenger):
     def __init__(self, count=1, discount_type='000', card='', card_no='', card_pw=''):
-        Passenger.__init__(self, '1', count, discount_type, card, card_no, card_pw)
+        Passenger.__init_internal__(self, '1', count, discount_type, card, card_no, card_pw)
 
 
 class ChildPassenger(Passenger):
     def __init__(self, count=1, discount_type='000', card='', card_no='', card_pw=''):
-        Passenger.__init__(self, '3', count, discount_type, card, card_no, card_pw)
+        Passenger.__init_internal__(self, '3', count, discount_type, card, card_no, card_pw)
 
 
 class SeniorPassenger(Passenger):
     def __init__(self, count=1, discount_type='P41', card='', card_no='', card_pw=''):
-        Passenger.__init__(self, '1', count, discount_type, card, card_no, card_pw)
+        Passenger.__init_internal__(self, '1', count, discount_type, card, card_no, card_pw)
+
+
+class TrainType:
+    KTX            = "00"  # "KTX, KTX-산천",
+    SAEMAEUL       = "01"  # "새마을호",
+    MUGUNGHWA      = "02"  # "무궁화호",
+    TONGGUEN       = "03"  # "통근열차",
+    NURIRO         = "04"  # "누리로",
+    ALL            = "05"  # "전체",
+    AIRPORT        = "06"  # "공항직통",
+    KTX_SANCHEON   = "KTX-07"  # "KTX-산천",
+    ITX_SAEMAEUL   = "ITX-08"  # "ITX-새마을",
+    ITX_CHEONGCHUN = "ITX-09"  # "ITX-청춘",
+
+    def __init__(self):
+        raise NotImplementedError("Do not make instance.")
+
+
+class ReserveOption:
+    GENERAL_FIRST = "GENERAL_FIRST" # 일반실 우선
+    GENERAL_ONLY = "GENERAL_ONLY"   # 일반실만
+    SPECIAL_FIRST = "SPECIAL_FIRST" # 특실 우선
+    SPECIAL_ONLY = "SPECIAL_ONLY"   # 특실만
+
+    def __init__(self):
+        raise NotImplementedError("Do not make instance.")
 
 
 class Reservation(Train):
@@ -483,7 +542,7 @@ class Korail(object):
     def _result_check(self, j):
         """Result data check"""
         if self.want_feedback:
-            print j['h_msg_txt']
+            print(j['h_msg_txt'])
 
         if j['strResult'] == 'FAIL':
             h_msg_cd  = j['h_msg_cd'].encode('utf-8')
@@ -497,8 +556,8 @@ class Korail(object):
         else:
             return True
 
-    def search_train(self, dep, arr, date=None, time=None, train_type='05',
-                     passengers=None):
+    def search_train(self, dep, arr, date=None, time=None, train_type=TrainType.ALL,
+                     passengers=None, show_all=False):
         """Search trains for specific time and date.
 
 :param dep: A departure station in Korean  ex) '서울'
@@ -516,6 +575,8 @@ class Korail(object):
                    - 07: KTX-산천
                    - 08: ITX-새마을
                    - 09: ITX-청춘
+:param passengers=None: (optional) List of Passenger Objects. None means 1 AdultPassenger.
+:param show_all=False: (optional) When True, a result includes trains which has no seats.
 """
         if date is None:
             date = datetime.now().strftime("%Y%m%d")
@@ -525,6 +586,8 @@ class Korail(object):
         if passengers is None:
             passengers = [AdultPassenger()]
 
+        passengers = Passenger.reduce(passengers)
+
         adult_count  = reduce(lambda a, b: a + b.count, filter(lambda x: isinstance(x, AdultPassenger), passengers), 0)
         child_count  = reduce(lambda a, b: a + b.count, filter(lambda x: isinstance(x, ChildPassenger), passengers), 0)
         senior_count = reduce(lambda a, b: a + b.count, filter(lambda x: isinstance(x, SeniorPassenger), passengers), 0)
@@ -568,18 +631,52 @@ class Korail(object):
                     try: info[i] = info[i].encode('utf-8')
                     except: pass
 
-                trains.append(Train(info))
+                train = Train(info)
+
+                if   show_all is True:
+                    trains.append(train)
+                elif show_all is False and train.has_seat():
+                    trains.append(train)
 
             return trains
 
-    def reserve(self, train, passengers=None):
+    def reserve(self, train, passengers=None, option=ReserveOption.GENERAL_FIRST):
         """Reserve a train.
 
 :param train: An instance of `Train`.
+:param passengers=None: (optional) List of Passenger Objects. None means 1 AdultPassenger.
+:param option=ReserveOption.: (optional)
         """
-        # train : 예약을 위한 차량의 필수 정보를 가진 모든 객체를 이용할 수 있어야 한다.
+
+        # 좌석 선택 옵션에 따라 결정.
+        seat_type = None
+        if train.has_seat() is False:                       # 자리가 둘다 없는 경우는 SoldOutError발생
+            raise SoldOutError()
+        elif option == ReserveOption.GENERAL_ONLY: # 이후 일반석, 특실 중 하나는 무조건 있는 조건
+            if train.has_general_seat():
+                seat_type = '1'
+            else:
+                raise SoldOutError()
+        elif option == ReserveOption.SPECIAL_ONLY:
+            if train.has_special_seat():
+                seat_type = '2'
+            else:
+                raise SoldOutError()
+        elif option == ReserveOption.GENERAL_FIRST:
+            if train.has_general_seat():
+                seat_type = '1'
+            else:
+                seat_type = '2'
+        elif option == ReserveOption.SPECIAL_FIRST:
+            if train.has_special_seat():
+                seat_type = '2'
+            else:
+                seat_type = '1'
+
         if passengers is None:
-            passengers = (AdultPassenger(),)
+            passengers = [AdultPassenger()]
+
+        passengers = Passenger.reduce(passengers)
 
         url = KORAIL_TICKETRESERVATION
         data = {
@@ -609,7 +706,7 @@ class Korail(object):
             'txtTrnNo1'      : train.train_no,
             'txtRunDt1'      : train.run_date,
             'txtTrnClsfCd1'  : train.train_type,
-            'txtPsrmClCd1'   : '1',
+            'txtPsrmClCd1'   : seat_type,
             'txtChgFlg1'     : '',
 
             # 이하 여정정보2
@@ -702,7 +799,7 @@ class Korail(object):
                         except: pass
                     reserves.append(Reservation(info))
                 return reserves
-        except NoResultsError, e:
+        except NoResultsError:
             return []
 
     def cancel(self, rsv):
